import { WellKnownMimeType, encodeRoute, encodeBearerAuthMetadata, encodeSimpleAuthMetadata, encodeCompositeMetadata, encodeAndAddWellKnownMetadata, CompositeMetadata, decodeRoutes } from 'rsocket-composite-metadata';
import { Buffer as Buffer$1 } from 'buffer';
import { RSocketConnector, ErrorCodes } from 'rsocket-core';
export { ErrorCodes, RSocketError } from 'rsocket-core';
import * as i0 from '@angular/core';
import { Injectable } from '@angular/core';
import { ReplaySubject, BehaviorSubject, retry, defer, first, map, switchMap } from 'rxjs';
import { WebsocketClientTransport } from 'rsocket-websocket-client';
import { print } from 'graphql/index';

class RsocketConfig {
    constructor(url) {
        this.config = {};
        this.payloadData = undefined;
        this.payloadDAuth = undefined;
        this._url = 'ws://localhost:8080/rs';
        this._url = url;
    }
    setup(setup) {
        this.config['setup'] = setup;
        return this;
    }
    data(data) {
        return this.payload(data, this.payloadDAuth);
    }
    auth(auth) {
        return this.payload(this.payloadData, auth);
    }
    payload(data, auth, route = 'setup') {
        this.payloadData = data;
        this.payloadDAuth = auth;
        if (!this.config.setup) {
            throw Error('payload() must after setup()');
        }
        this.config.setup['payload'] = {
            data: data ? Buffer$1.from(JSON.stringify(data)) : undefined,
            metadata: this.buildAuthMetadata({ route, auth })
        };
        return this;
    }
    fragmentation(fragmentation) {
        this.config['fragmentation'] = fragmentation;
        return this;
    }
    url(url) {
        this._url = url;
        return this;
    }
    getUrl() {
        return this._url;
    }
    // transport(transport: ConnectorConfig['transport']) {
    //     this.config['transport'] = transport
    //     return this
    // }
    responder(responder) {
        this.config['responder'] = responder;
        return this;
    }
    lease(lease) {
        this.config['lease'] = lease;
        return this;
    }
    resume(resume) {
        this.config['resume'] = resume;
        return this;
    }
    buildAuthMetadata({ route, auth }) {
        const map = new Map();
        map.set(WellKnownMimeType.MESSAGE_RSOCKET_ROUTING, encodeRoute(route));
        if (auth) {
            const { token, simple } = auth;
            if (token) {
                map.set(WellKnownMimeType.MESSAGE_RSOCKET_AUTHENTICATION, encodeBearerAuthMetadata(token));
            }
            else if (simple) {
                map.set(WellKnownMimeType.MESSAGE_RSOCKET_AUTHENTICATION, encodeSimpleAuthMetadata(simple.username, simple.password));
            }
        }
        return encodeCompositeMetadata(map);
    }
    build() {
        return this.config;
    }
}

var ConnectionStatus;
(function (ConnectionStatus) {
    ConnectionStatus[ConnectionStatus["DISCONNECTED"] = 0] = "DISCONNECTED";
    ConnectionStatus[ConnectionStatus["REJECT"] = 1] = "REJECT";
    ConnectionStatus[ConnectionStatus["CONNECTING"] = 2] = "CONNECTING";
    ConnectionStatus[ConnectionStatus["CONNECTED"] = 3] = "CONNECTED";
})(ConnectionStatus || (ConnectionStatus = {}));
var NetWorkStatus;
(function (NetWorkStatus) {
    NetWorkStatus[NetWorkStatus["IDLE"] = 0] = "IDLE";
    NetWorkStatus[NetWorkStatus["UPLOADING"] = 1] = "UPLOADING";
    NetWorkStatus[NetWorkStatus["DOWNLOADING"] = 2] = "DOWNLOADING";
    NetWorkStatus[NetWorkStatus["DONE"] = 3] = "DONE";
})(NetWorkStatus || (NetWorkStatus = {}));

let websocket;
const rsocketCreator = (url, { onSend, onMessage }) => {
    websocket?.close();
    websocket = new WebSocket(url);
    console.log('new');
    const originalSend = websocket.send;
    websocket.send = (data) => {
        onSend();
        originalSend.call(websocket, data);
    };
    const openHandler = () => {
        console.log('open12345');
    };
    const closeHandler = () => {
        websocket.removeEventListener('open', openHandler);
        websocket.removeEventListener('message', onMessage);
        websocket.removeEventListener('close', closeHandler);
    };
    websocket.addEventListener('open', openHandler);
    websocket.addEventListener('message', onMessage);
    websocket.addEventListener('close', closeHandler);
    return websocket;
};

class RsocketClient {
    constructor() {
        this._rsocketClient = new ReplaySubject(1);
        this._error = new ReplaySubject(1);
        this._connectionStatus = new BehaviorSubject(ConnectionStatus.DISCONNECTED);
        this._networkStatus = new BehaviorSubject(NetWorkStatus.IDLE);
        this._config = undefined;
    }
    connect() {
        this._connectionStatus.pipe(first()).subscribe(connectionStatus => {
            if (![ConnectionStatus.REJECT, ConnectionStatus.DISCONNECTED].includes(this._connectionStatus.getValue())) {
                return;
            }
            this._connectionStatus.next(ConnectionStatus.CONNECTING);
            if (!this._config) {
                throw new Error("rsocketConfig config is null");
            }
            const config = this._config.build();
            config['transport'] = new WebsocketClientTransport({
                url: this._config.getUrl(),
                wsCreator: (url) => rsocketCreator(url, {
                    onSend: () => this._networkStatus.next(NetWorkStatus.UPLOADING),
                    onMessage: () => this._networkStatus.next(NetWorkStatus.DOWNLOADING)
                })
            });
            const connector = new RSocketConnector(config);
            defer(() => connector.connect()).pipe(retry({ delay: 2000 })).subscribe({
                next: (rsocketClient) => {
                    console.log('Connected to RSocket server');
                    rsocketClient.onClose(error => {
                        const rsocketError = error;
                        if (rsocketError) {
                            this._error.next(rsocketError);
                            this._connectionStatus.next(ConnectionStatus.DISCONNECTED);
                            if (rsocketError.code === ErrorCodes.REJECTED_SETUP) {
                                this._connectionStatus.next(ConnectionStatus.REJECT);
                            }
                        }
                    });
                    this._rsocketClient.next(rsocketClient);
                },
                error: error => {
                    console.error('Failed to connect to RSocket server', error);
                }
            });
        });
    }
    encodePayload({ route, data, encodeData = true }) {
        let dataBuffer = null;
        if (data) {
            if (encodeData) {
                dataBuffer = Buffer.from(JSON.stringify(data));
            }
            else {
                dataBuffer = Buffer.from(data);
            }
        }
        return {
            data: dataBuffer,
            metadata: encodeAndAddWellKnownMetadata(Buffer.alloc(0), WellKnownMimeType.MESSAGE_RSOCKET_ROUTING, encodeRoute(route))
        };
    }
    fireAndForget({ payload }) {
        this._networkStatus.next(NetWorkStatus.UPLOADING);
        this._rsocketClient.pipe(map(rSocket => rSocket.fireAndForget(this.encodePayload(payload), {
            onError: (error) => {
            },
            onComplete: () => {
            }
        })), first()).subscribe(() => {
            this._networkStatus.next(NetWorkStatus.IDLE);
        });
    }
    requestResponse({ payload, responderStream }) {
        return this._rsocketClient.pipe(map(rSocket => rSocket.requestResponse(this.encodePayload(payload), responderStream)), first());
    }
    requestStream({ payload, initialRequestN, responderStream }) {
        return this._rsocketClient.pipe(map(rSocket => {
            return rSocket.requestStream(this.encodePayload(payload), initialRequestN, responderStream);
        }), first());
    }
    requestChannel({ payload, initialRequestN, isCompleted, responderStream }) {
        return this._rsocketClient.pipe(map(rSocket => rSocket.requestChannel(this.encodePayload(payload), initialRequestN, isCompleted, responderStream)), first());
    }
    get error() {
        return this._error.asObservable();
    }
    set config(rsocketConfig) {
        this._config = rsocketConfig;
    }
    get config() {
        if (!this._config) {
            throw new Error("rsocketConfig config is null");
        }
        return this._config;
    }
    get connectionStatus() {
        return this._connectionStatus;
    }
    updateNetworkState(state) {
        this._networkStatus.next(state);
    }
    get networkStatus() {
        return this._networkStatus.asObservable();
    }
}

class RsocketService {
    constructor() {
        this.rsocketClient = new RsocketClient();
    }
    connect() {
        this.rsocketClient.connect();
    }
    get connectionStatus() {
        return this.rsocketClient.connectionStatus;
    }
    get networkStatus() {
        return this.rsocketClient.networkStatus;
    }
    fireAndForget(payload) {
        this.rsocketClient.fireAndForget(payload);
    }
    requestChannel(params) {
        return this.rsocketClient.requestChannel(params);
    }
    setConfig(config) {
        this.rsocketClient.config = config;
    }
    setAuth(auth) {
        this.rsocketClient.config = this.rsocketClient.config.auth(auth);
    }
    setConnected() {
        this.rsocketClient.connectionStatus.next(ConnectionStatus.CONNECTED);
    }
    error() {
        return this.rsocketClient.error;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: RsocketService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: RsocketService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: RsocketService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

const routeHandleMap = new Map();
function mapMetaData(metadata) {
    const mappedMetaData = new Map();
    if (metadata) {
        for (let metaData of new CompositeMetadata(metadata)) {
            switch (metaData.mimeType) {
                case WellKnownMimeType.MESSAGE_RSOCKET_ROUTING.string: {
                    const tags = [];
                    let routes = decodeRoutes(metaData.content);
                    let entry = routes.next();
                    while (!entry.done) {
                        tags.push(entry.value);
                        entry = routes.next();
                    }
                    const joinedRoute = tags.join(".");
                    mappedMetaData.set(WellKnownMimeType.MESSAGE_RSOCKET_ROUTING.string, joinedRoute);
                }
            }
        }
    }
    return mappedMetaData;
}
function registerRoute({ route, handle, type = 'json' }) {
    switch (type) {
        case 'string':
            routeHandleMap.set(route, (payload) => handle(payload.data?.toString()));
            break;
        case 'json':
            routeHandleMap.set(route, (payload) => handle(payload.data));
    }
}
function deleteRoute(route) {
    if (typeof route === 'string') {
        routeHandleMap.delete(route);
    }
    else {
        Array.from(routeHandleMap.keys()).forEach(key => {
            if (route.test(key)) {
                routeHandleMap.delete(key);
            }
        });
    }
}
function handleFireAndForget(payload) {
    const route = mapMetaData(payload.metadata).get(WellKnownMimeType.MESSAGE_RSOCKET_ROUTING.string);
    const handle = routeHandleMap.get(route);
    if (!handle) {
        console.error(`can not find route for ${route}`);
    }
    else {
        handle(payload);
    }
}

class GraphqlClient {
    constructor(rsocketClient, route) {
        this.rsocketClient = rsocketClient;
        this.route = route;
    }
    query(query) {
        const { documentNode, extract, variables } = query;
        const result = new ReplaySubject(1);
        return this.rsocketClient.requestResponse({
            payload: { data: { query: print(documentNode), variables }, route: this.route },
            responderStream: {
                onNext: (payload, isComplete) => {
                    result.next(extract(payload));
                    this.rsocketClient.updateNetworkState(NetWorkStatus.DONE);
                },
                onComplete() {
                },
                onError(error) {
                    console.log(error);
                },
                onExtension(extendedType, content, canBeIgnored) {
                }
            }
        }).pipe(switchMap(() => result), first());
    }
    subscription(subscription) {
        const { documentNode, extract, variables } = subscription;
        const result = new ReplaySubject();
        const isCompleted = new BehaviorSubject(false);
        const error = new ReplaySubject();
        return this.rsocketClient.requestStream({
            payload: { data: { query: print(documentNode), variables }, route: this.route },
            initialRequestN: 1,
            responderStream: {
                onNext: (payload, isComplete) => {
                    result.next(extract(payload));
                },
                onComplete: () => {
                    this.rsocketClient.updateNetworkState(NetWorkStatus.DONE);
                },
                onError(e) {
                    error.next(e);
                },
                onExtension(extendedType, content, canBeIgnored) {
                }
            }
        }).pipe(map(requester => {
            return { result, requester, isCompleted, error };
        }), first());
    }
    mutation(mutation) {
        const { documentNode, extract, variables } = mutation;
        const result = new ReplaySubject(1);
        return this.rsocketClient.requestResponse({
            payload: { data: { query: print(documentNode), variables }, route: this.route },
            responderStream: {
                onNext: (payload, isComplete) => {
                    result.next(extract(payload));
                    this.rsocketClient.updateNetworkState(NetWorkStatus.DONE);
                },
                onComplete() {
                },
                onError(error) {
                    console.log(error);
                },
                onExtension(extendedType, content, canBeIgnored) {
                }
            }
        }).pipe(switchMap(() => result), first());
    }
}

class GraphqlService {
    constructor(rsocketService) {
        this.rsocketService = rsocketService;
        this._graphqlClient = {};
        this._graphqlClient = new GraphqlClient(this.rsocketService.rsocketClient, 'graphql');
    }
    query(query) {
        return this._graphqlClient.query(query);
    }
    subscription(subscription) {
        return this._graphqlClient.subscription(subscription);
    }
    mutation(mutation) {
        return this._graphqlClient.mutation(mutation);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: GraphqlService, deps: [{ token: RsocketService }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: GraphqlService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.1.0", ngImport: i0, type: GraphqlService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: RsocketService }] });

var Operation;
(function (Operation) {
    Operation["QUERY"] = "query";
    Operation["SUBSCRIPTION"] = "subscription";
    Operation["MUTATION"] = "mutation";
})(Operation || (Operation = {}));
function create(documentNode, extract, operation, variables) {
    const valid = documentNode.definitions.some(definition => {
        //@ts-ignore
        return definition.selectionSet.selections.some(selection => {
            if ("name" in selection && selection.name.value === extract) {
                //@ts-ignore
                if (definition.operation !== operation) {
                    throw Error(`createGraphqlExtractor=> '${operation}' type error ${documentNode.loc?.source.body}`);
                }
                return true;
            }
            return false;
        });
    });
    if (!valid) {
        throw Error(`createGraphqlExtractor=> '${extract}' not in ${documentNode.loc?.source.body}`);
    }
    return {
        documentNode,
        extract: (payload) => {
            if (!payload.data) {
                throw Error(`GraphqlExtractor => payload.data is null, extract: ${extract}`);
            }
            const value = JSON.parse(payload.data.toString());
            if (value['data'] !== undefined && value['data'][extract] !== undefined) {
                return value['data'][extract];
            }
            throw Error(`GraphqlExtractor => 'data' or '${extract}' not in ${JSON.stringify(value)}`);
        },
        variables
    };
}
function createQuery(documentNode, extract, variables) {
    return create(documentNode, extract, Operation.QUERY, variables);
}
function createSubscription(documentNode, extract, variables) {
    return create(documentNode, extract, Operation.SUBSCRIPTION, variables);
}
function createMutation(documentNode, extract, variables) {
    return create(documentNode, extract, Operation.MUTATION, variables);
}

/*
 * Public API Surface of ngx-rsocket-graphql
 */
window.Buffer = Buffer$1;

/**
 * Generated bundle index. Do not edit.
 */

export { ConnectionStatus, GraphqlService, NetWorkStatus, RsocketConfig, RsocketService, createMutation, createQuery, createSubscription, deleteRoute, handleFireAndForget, registerRoute };
//# sourceMappingURL=ngx-rsocket-graphql.mjs.map
